@@ -1,3 +1,25 @@
+/*
+ * AT-SPI - Assistive Technology Service Provider Interface
+ * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
+ *
+ * Copyright 2001 Sun Microsystems Inc.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
 #include <cspi/spi-private.h>
 
 /**
@@ -66,7 +88,7 @@ AccessibleAction_getDescription (AccessibleAction *obj,
 
   retval =
     Accessibility_Action_getDescription (CSPI_OBJREF (obj),
-					 (CORBA_long) i,
+					 i,
 					 cspi_ev ());
 
   cspi_return_val_if_ev ("getDescription", NULL);
@@ -95,7 +117,7 @@ AccessibleAction_getKeyBinding (AccessibleAction *obj,
 
   retval =
     Accessibility_Action_getKeyBinding (CSPI_OBJREF (obj),
-       (CORBA_long) i,
+       i,
        cspi_ev ());
 
   cspi_return_val_if_ev ("getKeyBinding", NULL);
@@ -123,7 +145,7 @@ AccessibleAction_getName (AccessibleAction *obj,
 
   retval =
    Accessibility_Action_getName (CSPI_OBJREF (obj),
-				 (CORBA_long) i,
+				 i,
 				 cspi_ev ());
 
   cspi_return_val_if_ev ("getName", NULL);
@@ -148,9 +170,8 @@ AccessibleAction_doAction (AccessibleAction *obj,
 
   cspi_return_val_if_fail (obj != NULL, FALSE);
 
-  retval = (SPIBoolean)
-    Accessibility_Action_doAction (CSPI_OBJREF (obj),
-				   (CORBA_long) i,
+  retval = Accessibility_Action_doAction (CSPI_OBJREF (obj),
+				   i,
 				   cspi_ev ());
 
   cspi_return_val_if_ev ("doAction", FALSE);
